@@ -49,6 +49,11 @@ class Server(object):
                            connect handler and your client is confused when it
                            receives events before the connection acceptance.
                            In any other case use the default of ``False``.
+    :param namespaces: a list of namespaces that are accepted, in addition to
+                       any namespaces for which handlers have been defined. The
+                       default is `['/']`, which always accepts connections to
+                       the default namespace. Set to `'*'` to accept all
+                       namespaces.
     :param kwargs: Connection parameters for the underlying Engine.IO server.
 
     The Engine.IO configuration supports the following settings:
@@ -110,7 +115,7 @@ class Server(object):
 
     def __init__(self, client_manager=None, logger=False, serializer='default',
                  json=None, async_handlers=True, always_connect=False,
-                 **kwargs):
+                 namespaces=None, **kwargs):
         engineio_options = kwargs
         engineio_logger = engineio_options.pop('engineio_logger', None)
         if engineio_logger is not None:
@@ -134,6 +139,7 @@ class Server(object):
         self.environ = {}
         self.handlers = {}
         self.namespace_handlers = {}
+        self.not_handled = object()
 
         self._binary_packet = {}
 
@@ -156,6 +162,7 @@ class Server(object):
 
         self.async_handlers = async_handlers
         self.always_connect = always_connect
+        self.namespaces = namespaces or ['/']
 
         self.async_mode = self.eio.async_mode
 
@@ -262,7 +269,7 @@ class Server(object):
             namespace_handler
 
     def emit(self, event, data=None, to=None, room=None, skip_sid=None,
-             namespace=None, callback=None, **kwargs):
+             namespace=None, callback=None, ignore_queue=False):
         """Emit a custom event to one or more connected clients.
 
         :param event: The event name. It can be any string. The event names
@@ -274,7 +281,7 @@ class Server(object):
                      one of the types indicated above.
         :param to: The recipient of the message. This can be set to the
                    session ID of a client to address only that client, to any
-                   any custom room created by the application to address all
+                   custom room created by the application to address all
                    the clients in that room, or to a list of custom room
                    names. If this argument is omitted the event is broadcasted
                    to all connected clients.
@@ -287,7 +294,7 @@ class Server(object):
                           argument is omitted the event is emitted to the
                           default namespace.
         :param callback: If given, this function will be called to acknowledge
-                         the the client has received the message. The arguments
+                         the client has received the message. The arguments
                          that will be passed to the function are those provided
                          by the client. Callback functions can only be used
                          when addressing an individual client.
@@ -310,10 +317,11 @@ class Server(object):
         self.logger.info('emitting event "%s" to %s [%s]', event,
                          room or 'all', namespace)
         self.manager.emit(event, data, namespace, room=room,
-                          skip_sid=skip_sid, callback=callback, **kwargs)
+                          skip_sid=skip_sid, callback=callback,
+                          ignore_queue=ignore_queue)
 
     def send(self, data, to=None, room=None, skip_sid=None, namespace=None,
-             callback=None, **kwargs):
+             callback=None, ignore_queue=False):
         """Send a message to one or more connected clients.
 
         This function emits an event with the name ``'message'``. Use
@@ -338,7 +346,7 @@ class Server(object):
                           argument is omitted the event is emitted to the
                           default namespace.
         :param callback: If given, this function will be called to acknowledge
-                         the the client has received the message. The arguments
+                         the client has received the message. The arguments
                          that will be passed to the function are those provided
                          by the client. Callback functions can only be used
                          when addressing an individual client.
@@ -351,10 +359,11 @@ class Server(object):
                              value of ``False``.
         """
         self.emit('message', data=data, to=to, room=room, skip_sid=skip_sid,
-                  namespace=namespace, callback=callback, **kwargs)
+                  namespace=namespace, callback=callback,
+                  ignore_queue=ignore_queue)
 
     def call(self, event, data=None, to=None, sid=None, namespace=None,
-             timeout=60, **kwargs):
+             timeout=60, ignore_queue=False):
         """Emit a custom event to a client and wait for the response.
 
         This method issues an emit with a callback and waits for the callback
@@ -405,7 +414,7 @@ class Server(object):
             callback_event.set()
 
         self.emit(event, data=data, room=to or sid, namespace=namespace,
-                  callback=event_callback, **kwargs)
+                  callback=event_callback, ignore_queue=ignore_queue)
         if not callback_event.wait(timeout=timeout):
             raise exceptions.TimeoutError()
         return callback_args[0] if len(callback_args[0]) > 1 \
@@ -558,7 +567,8 @@ class Server(object):
             self._send_packet(eio_sid, self.packet_class(
                 packet.DISCONNECT, namespace=namespace))
             self._trigger_event('disconnect', namespace, sid)
-            self.manager.disconnect(sid, namespace=namespace)
+            self.manager.disconnect(sid, namespace=namespace,
+                                    ignore_queue=True)
 
     def transport(self, sid):
         """Return the name of the transport used by the client.
@@ -648,7 +658,10 @@ class Server(object):
     def _handle_connect(self, eio_sid, namespace, data):
         """Handle a client connection request."""
         namespace = namespace or '/'
-        sid = self.manager.connect(eio_sid, namespace)
+        sid = None
+        if namespace in self.handlers or namespace in self.namespace_handlers \
+                or self.namespaces == '*' or namespace in self.namespaces:
+            sid = self.manager.connect(eio_sid, namespace)
         if sid is None:
             self._send_packet(eio_sid, self.packet_class(
                 packet.CONNECT_ERROR, data='Unable to connect',
@@ -683,7 +696,7 @@ class Server(object):
                 self._send_packet(eio_sid, self.packet_class(
                     packet.CONNECT_ERROR, data=fail_reason,
                     namespace=namespace))
-            self.manager.disconnect(sid, namespace)
+            self.manager.disconnect(sid, namespace, ignore_queue=True)
         elif not self.always_connect:
             self._send_packet(eio_sid, self.packet_class(
                 packet.CONNECT, {'sid': sid}, namespace=namespace))
@@ -696,7 +709,7 @@ class Server(object):
             return
         self.manager.pre_disconnect(sid, namespace=namespace)
         self._trigger_event('disconnect', namespace, sid)
-        self.manager.disconnect(sid, namespace)
+        self.manager.disconnect(sid, namespace, ignore_queue=True)
 
     def _handle_event(self, eio_sid, namespace, id, data):
         """Handle an incoming client event."""
@@ -718,7 +731,7 @@ class Server(object):
     def _handle_event_internal(self, server, sid, eio_sid, data, namespace,
                                id):
         r = server._trigger_event(data[0], namespace, sid, *data[1:])
-        if id is not None:
+        if r != self.not_handled and id is not None:
             # send ACK packet with the response returned by the handler
             # tuples are expanded as multiple arguments
             if r is None:
@@ -746,9 +759,11 @@ class Server(object):
             elif event not in self.reserved_events and \
                     '*' in self.handlers[namespace]:
                 return self.handlers[namespace]['*'](event, *args)
+            else:
+                return self.not_handled
 
         # or else, forward the event to a namespace handler if one exists
-        elif namespace in self.namespace_handlers:
+        elif namespace in self.namespace_handlers:  # pragma: no branch
             return self.namespace_handlers[namespace].trigger_event(
                 event, *args)
 
